#import "Program.h"
#import <stdio.h>

@implementation Program


-(Program*)init {
	[ super init ];
	self->title       = nil;
	self->startTime   = 0;
	self->endTime     = 0;
	self->description = nil;
	self->comment     = nil;
	self->genre       = 0;
	self->g_code      = -1;
	self->address_epg = @"";
	return self;
}

-(Program*)initWithCSVString: (NSString*)row {
	int i;
	int len = [ row length ];
	int itemnum = 0;
	int pos = 0;
	int duration = 0;
	
	[ self init ];
	for ( i = 0; i <= len ; i ++ ) {
		unichar u;
		if ( i == len ) {
			u = L',';
		} else {
			u = [ row characterAtIndex: i ];
		}
		if ( u == L',' ) {//|| u == L'\0' ) {
			NSRange range;
			NSString* datum;
			range.location = pos;
			range.length   = i - pos;
			pos = i + 1;
			//i ++;
			datum = [ row substringWithRange: range ];
			switch ( itemnum ) {
				case 0:
					startTime = [ datum intValue ];
					break;
				case 1:
					duration = [ datum intValue ];
					break;
				case 2:
					[ self setTitle: datum ];
					break;
				case 3:
					[ self setDescription: datum ];
					break;
				case 4:
					[ self setComment: datum ];
					break;
				case 5:
					genre = [ datum intValue ];
					break;
				case 6:
					g_code = [ datum intValue ];
					break;
				case 7:
					[ self setEPGAddress: datum ];
					break;
			}
			itemnum ++;
			if (itemnum >= 8 ) break;
		}
  }
	endTime = startTime + duration;
	return self;
}

-(void)dealloc {
	[ super dealloc ];
	[ title release ];
	[ description release ];
	[ comment release ];
}

-(void)setTime : (int) start minutes: (int) duration {
	startTime = start;
	endTime = start + duration;
}

-(int)getStartTime {
	return startTime;
}

-(int)getEndTime {
	return endTime;
}

-(int)getDuration {
	return endTime - startTime;
}

-(void)setTitle : (NSString*) aTitle {
	[ title release ];
	title = aTitle;
	[ title retain ];
}

-(void)setDescription : (NSString*) aDescription {
	[ description release ];
	description = aDescription;
	[ description retain ];
}

-(void)setComment : (NSString*) aComment {
	[ comment release ];
	comment = aComment;
	[ comment retain ];
}


-(NSString*)getTitle {
	return title;
}

-(NSString*)getDescription {
	return description;
}

-(NSString*)getComment {
	return comment;
}

-(void)setEPGAddress: (NSString*)url {
	[ address_epg release ];
	address_epg = url;
	[ address_epg retain ];
}

-(NSString*)getEPGAddress {
	return address_epg;
}

-(int)getGenre {
	return genre;
}

-(void)setGenre : (int)theGenre {
	genre = theGenre;
}

+(NSString*)getGenreName : (int) genreCode {
	switch ( genreCode ) {
		case 1: return @"food";
		case 2: return @"drama";
		case 3: return @"sports";
		case 4: return @"anime";
		case 5: return @"music";
		case 6: return @"movie";
	}
	return @"general";
}

-(void)setGCode : (int) code {
	g_code = code;
}

-(int)getGCode {
	return g_code;
}

-(NSString*)description {
	NSString* output = [ NSString stringWithFormat:@"%d,%d,%@,%@,%@,%d", startTime, endTime - startTime, title, description, comment, genre ];
	return output;
}

+(void) replaceWithPattern:(NSMutableString*)original pattern:(NSString*)pattern to:(NSString*)replaced {
	NSRange range;
	NSRange range_next;
	range = [ original rangeOfString: pattern ];
	while ( range.location != NSNotFound ) {
		[ original replaceCharactersInRange : range withString : replaced ];
		range_next.location = range.location;
		range_next.length = [ original length ] - range_next.location;
		range = [ original rangeOfString: pattern options:0 range:range_next ];
		
	}
}

+(NSString*)getTimeString: (int)minutes {
	int hr = minutes / 60;
	int mn = minutes % 60;
	NSString* hs;
	NSString* ms;
	if ( hr < 10 ) {
		hs = [ NSString stringWithFormat: @"0%d", hr ];
	} else {
		hs = [ NSString stringWithFormat: @"%d", hr ];
	}
	if ( mn < 10 ) {
		ms = [ NSString stringWithFormat: @"0%d", mn ];
	} else {
		ms = [ NSString stringWithFormat: @"%d", mn ];
	}
	return [ NSString stringWithFormat: @"%@:%@", hs, ms ];
}


-(NSString*)toHTML : (NSString*)format {
	NSString* rep;
	NSString* org;
	NSMutableString* output = [ NSMutableString stringWithString: format ];
	//NSLog( @"%@", output );
	
	// Genre
	org = @"<$genre$>";
	rep = [ Program getGenreName:genre ];
	[ Program replaceWithPattern:output pattern:org to:rep ];

	// Title
	org = @"<$title$>";
	rep = title;
	[ Program replaceWithPattern:output pattern:org to: rep ];

	// Comment
	org = @"<$comment$>";
	rep = comment;
	[ Program replaceWithPattern: output pattern: org to: rep ];

	// Description
	org = @"<$description$>";
	rep = description;
	[ Program replaceWithPattern: output pattern: org to: rep ];

	// Length of program
	org = @"<$length$>";
	rep = [ NSString stringWithFormat: @"%d", endTime - startTime ];
	[ Program replaceWithPattern: output pattern: org to: rep ];

	// time table
	org = @"<$starttime$>";
	{
		int st_24 = startTime;
		if ( st_24 > 1440 ) st_24 -= 1440;
		rep = [ Program getTimeString : st_24 ];
	}
	[ Program replaceWithPattern: output pattern: org to: rep ];
	
	org = @"<$endtime$>";
	{
		int ed_24 = endTime;
		if ( ed_24 > 1440 ) ed_24 -= 1440;
		rep = [ Program getTimeString : ed_24 ];
	}
	[ Program replaceWithPattern: output pattern: org to: rep ];
	//NSLog( @"%@", output );
	
	// G code
	org = @"<$gcode$>";
	if ( g_code >= 0 ) {
		rep = [ NSString stringWithFormat: @"%d", g_code ];
		[ Program replaceWithPattern: output pattern:org to: rep ];
	} else {
		[ Program replaceWithPattern: output pattern:org to: @"" ];
	}
	
	// iEPG
	org = @"<$iepg$>";
	[ Program replaceWithPattern: output pattern:org to:address_epg ];
	
	return output;
}


@end

int _Program__comparator( id p1, id p2, void* context ) {
	return [ (Program*)p1 getStartTime ] - [ (Program*)p2 getStartTime ];
}
